Method: ActiveSupport::Cache::MemCacheStore#initialize

Defined in:
lib/active_support/cache/mem_cache_store.rb

#initialize(*addresses) ⇒ MemCacheStore

Creates a new MemCacheStore object, with the given memcached server addresses. Each address is either a host name, or a host-with-port string in the form of “host_name:port”. For example:

ActiveSupport::Cache::MemCacheStore.new("localhost", "server-downstairs.localnetwork:8229")

If no addresses are specified, then MemCacheStore will connect to localhost port 11211 (the default memcached port).

Instead of addresses one can pass in a MemCache-like object. For example:

require 'memcached' # gem install memcached; uses C bindings to libmemcached
ActiveSupport::Cache::MemCacheStore.new(Memcached::Rails.new("localhost:11211"))


56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
# File 'lib/active_support/cache/mem_cache_store.rb', line 56

def initialize(*addresses)
  addresses = addresses.flatten
  options = addresses.extract_options!
  super(options)

  if addresses.first.respond_to?(:get)
    @data = addresses.first
  else
    mem_cache_options = options.dup
    UNIVERSAL_OPTIONS.each{|name| mem_cache_options.delete(name)}
    @data = self.class.build_mem_cache(*(addresses + [mem_cache_options]))
  end

  extend Strategy::LocalCache
  extend LocalCacheWithRaw
end