Module: ActiveSupport::CompareWithRange
- Included in:
- Range
- Defined in:
- lib/active_support/core_ext/range/compare_range.rb
Instance Method Summary collapse
-
#===(value) ⇒ Object
Extends the default Range#=== to support range comparisons.
-
#include?(value) ⇒ Boolean
Extends the default Range#include? to support range comparisons.
Instance Method Details
#===(value) ⇒ Object
Extends the default Range#=== to support range comparisons. (1..5) === (1..5) # => true (1..5) === (2..3) # => true (1..5) === (1...6) # => true (1..5) === (2..6) # => false
The native Range#=== behavior is untouched. ('a'..'f') === ('c') # => true (5..9) === (11) # => false
The given range must be fully bounded, with both start and end.
16 17 18 19 20 21 22 |
# File 'lib/active_support/core_ext/range/compare_range.rb', line 16 def ===(value) if value.is_a?(::Range) cover?(value) else super end end |
#include?(value) ⇒ Boolean
Extends the default Range#include? to support range comparisons. (1..5).include?(1..5) # => true (1..5).include?(2..3) # => true (1..5).include?(1...6) # => true (1..5).include?(2..6) # => false
The native Range#include? behavior is untouched. ('a'..'f').include?('c') # => true (5..9).include?(11) # => false
The given range must be fully bounded, with both start and end.
35 36 37 38 39 40 41 |
# File 'lib/active_support/core_ext/range/compare_range.rb', line 35 def include?(value) if value.is_a?(::Range) cover?(value) else super end end |