Class: ActiveSupport::MessageVerifier
- Inherits:
-
ActiveSupport::Messages::Codec
- Object
- ActiveSupport::Messages::Codec
- ActiveSupport::MessageVerifier
- Includes:
- ActiveSupport::Messages::Rotator
- Defined in:
- lib/active_support/message_verifier.rb
Overview
Active Support Message Verifier
MessageVerifier makes it easy to generate and verify messages which are
signed to prevent tampering.
In a Rails application, you can use Rails.application.message_verifier
to manage unique instances of verifiers for each use case.
Learn more.
This is useful for cases like remember-me tokens and auto-unsubscribe links where the session store isn't suitable or available.
First, generate a signed message:
[:remember_me] = Rails.application.(:remember_me).generate([@user.id, 2.weeks.from_now])
Later verify that message:
id, time = Rails.application.(:remember_me).verify([:remember_me])
if time.future?
self.current_user = User.find(id)
end
Confine messages to a specific purpose
It's not recommended to use the same verifier for different purposes in your application.
Doing so could allow a malicious actor to re-use a signed message to perform an unauthorized
action.
You can reduce this risk by confining signed messages to a specific :purpose.
token = @verifier.generate("signed message", purpose: :login)
Then that same purpose must be passed when verifying to get the data back out:
@verifier.verified(token, purpose: :login) # => "signed message"
@verifier.verified(token, purpose: :shipping) # => nil
@verifier.verified(token) # => nil
@verifier.verify(token, purpose: :login) # => "signed message"
@verifier.verify(token, purpose: :shipping) # => raises ActiveSupport::MessageVerifier::InvalidSignature
@verifier.verify(token) # => raises ActiveSupport::MessageVerifier::InvalidSignature
Likewise, if a message has no purpose it won't be returned when verifying with a specific purpose.
token = @verifier.generate("signed message")
@verifier.verified(token, purpose: :redirect) # => nil
@verifier.verified(token) # => "signed message"
@verifier.verify(token, purpose: :redirect) # => raises ActiveSupport::MessageVerifier::InvalidSignature
@verifier.verify(token) # => "signed message"
Expiring messages
By default messages last forever and verifying one year from now will still
return the original value. But messages can be set to expire at a given
time with :expires_in or :expires_at.
@verifier.generate("signed message", expires_in: 1.month)
@verifier.generate("signed message", expires_at: Time.now.end_of_year)
Messages can then be verified and returned until expiry.
Thereafter, the verified method returns nil while verify raises
ActiveSupport::MessageVerifier::InvalidSignature.
Rotating keys
MessageVerifier also supports rotating out old configurations by falling
back to a stack of verifiers. Call rotate to build and add a verifier so
either verified or verify will also try verifying with the fallback.
By default any rotated verifiers use the values of the primary verifier unless specified otherwise.
You'd give your verifier the new defaults:
verifier = ActiveSupport::MessageVerifier.new(@secret, digest: "SHA512", serializer: JSON)
Then gradually rotate the old values out by adding them as fallbacks. Any message generated with the old values will then work until the rotation is removed.
verifier.rotate(old_secret) # Fallback to an old secret instead of @secret.
verifier.rotate(digest: "SHA256") # Fallback to an old digest instead of SHA512.
verifier.rotate(serializer: Marshal) # Fallback to an old serializer instead of JSON.
Though the above would most likely be combined into one rotation:
verifier.rotate(old_secret, digest: "SHA256", serializer: Marshal)
Defined Under Namespace
Classes: InvalidSignature
Constant Summary collapse
- SEPARATOR =
:nodoc:
"--"- SEPARATOR_LENGTH =
:nodoc:
SEPARATOR.length
Constants included from ActiveSupport::Messages::Metadata
ActiveSupport::Messages::Metadata::ENVELOPE_SERIALIZERS, ActiveSupport::Messages::Metadata::TIMESTAMP_SERIALIZERS
Instance Attribute Summary
Attributes included from ActiveSupport::Messages::Metadata
#use_message_serializer_for_metadata
Instance Method Summary collapse
-
#create_message(value, **options) ⇒ Object
:nodoc:.
-
#generate(value, **options) ⇒ Object
Generates a signed message for the provided value.
-
#initialize(secret, **options) ⇒ MessageVerifier
constructor
Initialize a new MessageVerifier with a secret for the signature.
-
#inspect ⇒ Object
:nodoc:.
-
#read_message(message, **options) ⇒ Object
:nodoc:.
-
#valid_message?(message) ⇒ Boolean
Checks if a signed message could have been generated by signing an object with the +MessageVerifier+'s secret.
-
#verified(message, **options) ⇒ Object
Decodes the signed message using the +MessageVerifier+'s secret.
-
#verify(message, **options) ⇒ Object
Decodes the signed message using the +MessageVerifier+'s secret.
Methods included from ActiveSupport::Messages::Rotator
Constructor Details
#initialize(secret, **options) ⇒ MessageVerifier
Initialize a new MessageVerifier with a secret for the signature.
Options
[+:digest+]
Digest used for signing. The default is "SHA1". See
OpenSSL::Digest for alternatives.
[+:serializer+]
The serializer used to serialize message data. You can specify any
object that responds to dump and load, or you can choose from
several preconfigured serializers: :marshal, :json_allow_marshal,
:json, :message_pack_allow_marshal, :message_pack.
The preconfigured serializers include a fallback mechanism to support
multiple deserialization formats. For example, the +:marshal+ serializer
will serialize using +Marshal+, but can deserialize using +Marshal+,
ActiveSupport::JSON, or ActiveSupport::MessagePack. This makes it easy
to migrate between serializers.
The +:marshal+, +:json_allow_marshal+, and +:message_pack_allow_marshal+
serializers support deserializing using +Marshal+, but the others do
not. Beware that +Marshal+ is a potential vector for deserialization
attacks in cases where a message signing secret has been leaked. <em>If
possible, choose a serializer that does not support +Marshal+.</em>
The +:message_pack+ and +:message_pack_allow_marshal+ serializers use
ActiveSupport::MessagePack, which can roundtrip some Ruby types that are
not supported by JSON, and may provide improved performance. However,
these require the +msgpack+ gem.
When using \Rails, the default depends on +config.active_support.message_serializer+.
Otherwise, the default is +:marshal+.
[+:url_safe+]
By default, MessageVerifier generates RFC 4648 compliant strings which are
not URL-safe. In other words, they can contain "+" and "/". If you want to
generate URL-safe strings (in compliance with "Base 64 Encoding with URL
and Filename Safe Alphabet" in RFC 4648), you can pass true.
[+:force_legacy_metadata_serializer+] Whether to use the legacy metadata serializer, which serializes the message first, then wraps it in an envelope which is also serialized. This was the default in Rails 7.0 and below.
If you don't pass a truthy value, the default is set using
+config.active_support.use_message_serializer_for_metadata+.
153 154 155 156 157 158 |
# File 'lib/active_support/message_verifier.rb', line 153 def initialize(secret, **) raise ArgumentError, "Secret should not be nil." unless secret super(**) @secret = secret @digest = [:digest]&.to_s || "SHA1" end |
Instance Method Details
#create_message(value, **options) ⇒ Object
:nodoc:
296 297 298 |
# File 'lib/active_support/message_verifier.rb', line 296 def (value, **) # :nodoc: sign_encoded(encode((value, **))) end |
#generate(value, **options) ⇒ Object
Generates a signed message for the provided value.
The message is signed with the +MessageVerifier+'s secret. Returns Base64-encoded message joined with the generated signature.
verifier = ActiveSupport::MessageVerifier.new("secret")
verifier.generate("signed message") # => "BAhJIhNzaWduZWQgbWVzc2FnZQY6BkVU--f67d5f27c3ee0b8483cebf2103757455e947493b"
Options
[+:expires_at+] The datetime at which the message expires. After this datetime, verification of the message will fail.
= verifier.generate("hello", expires_at: Time.now.tomorrow)
verifier.verified() # => "hello"
# 24 hours later...
verifier.verified() # => nil
verifier.verify() # => raises ActiveSupport::MessageVerifier::InvalidSignature
[+:expires_in+] The duration for which the message is valid. After this duration has elapsed, verification of the message will fail.
= verifier.generate("hello", expires_in: 24.hours)
verifier.verified() # => "hello"
# 24 hours later...
verifier.verified() # => nil
verifier.verify() # => raises ActiveSupport::MessageVerifier::InvalidSignature
[+:purpose+] The purpose of the message. If specified, the same purpose must be specified when verifying the message; otherwise, verification will fail. (See #verified and #verify.)
292 293 294 |
# File 'lib/active_support/message_verifier.rb', line 292 def generate(value, **) (value, **) end |
#inspect ⇒ Object
:nodoc:
304 305 306 |
# File 'lib/active_support/message_verifier.rb', line 304 def inspect # :nodoc: "#<#{self.class.name}:#{'%#016x' % (object_id << 1)}>" end |
#read_message(message, **options) ⇒ Object
:nodoc:
300 301 302 |
# File 'lib/active_support/message_verifier.rb', line 300 def (, **) # :nodoc: (decode(extract_encoded()), **) end |
#valid_message?(message) ⇒ Boolean
Checks if a signed message could have been generated by signing an object with the +MessageVerifier+'s secret.
verifier = ActiveSupport::MessageVerifier.new("secret")
= verifier.generate("signed message")
verifier.() # => true
= .chop # editing the message invalidates the signature
verifier.() # => false
169 170 171 |
# File 'lib/active_support/message_verifier.rb', line 169 def () !!catch_and_ignore(:invalid_message_format) { extract_encoded() } end |
#verified(message, **options) ⇒ Object
Decodes the signed message using the +MessageVerifier+'s secret.
verifier = ActiveSupport::MessageVerifier.new("secret")
= verifier.generate("signed message")
verifier.verified() # => "signed message"
Returns nil if the message was not signed with the same secret.
other_verifier = ActiveSupport::MessageVerifier.new("different_secret")
other_verifier.verified() # => nil
Returns nil if the message is not Base64-encoded.
= "f--46a0120593880c733a53b6dad75b42ddc1c8996d"
verifier.verified() # => nil
Raises any error raised while decoding the signed message.
= "test--dad7b06c94abba8d46a15fafaef56c327665d5ff"
verifier.verified() # => TypeError: incompatible marshal file format
Options
[+:purpose+]
The purpose that the message was generated with. If the purpose does not
match, verified will return nil.
= verifier.generate("hello", purpose: "greeting")
verifier.verified(, purpose: "greeting") # => "hello"
verifier.verified(, purpose: "chatting") # => nil
verifier.verified() # => nil
= verifier.generate("bye")
verifier.verified() # => "bye"
verifier.verified(, purpose: "greeting") # => nil
210 211 212 213 214 215 216 217 218 |
# File 'lib/active_support/message_verifier.rb', line 210 def verified(, **) catch_and_ignore :invalid_message_format do catch_and_raise :invalid_message_serialization do catch_and_ignore :invalid_message_content do (, **) end end end end |
#verify(message, **options) ⇒ Object
Decodes the signed message using the +MessageVerifier+'s secret.
verifier = ActiveSupport::MessageVerifier.new("secret")
= verifier.generate("signed message")
verifier.verify() # => "signed message"
Raises InvalidSignature if the message was not signed with the same
secret or was not Base64-encoded.
other_verifier = ActiveSupport::MessageVerifier.new("different_secret")
other_verifier.verify() # => ActiveSupport::MessageVerifier::InvalidSignature
Options
[+:purpose+]
The purpose that the message was generated with. If the purpose does not
match, verify will raise ActiveSupport::MessageVerifier::InvalidSignature.
= verifier.generate("hello", purpose: "greeting")
verifier.verify(, purpose: "greeting") # => "hello"
verifier.verify(, purpose: "chatting") # => raises InvalidSignature
verifier.verify() # => raises InvalidSignature
= verifier.generate("bye")
verifier.verify() # => "bye"
verifier.verify(, purpose: "greeting") # => raises InvalidSignature
248 249 250 251 252 253 254 255 256 |
# File 'lib/active_support/message_verifier.rb', line 248 def verify(, **) catch_and_raise :invalid_message_format, as: InvalidSignature do catch_and_raise :invalid_message_serialization do catch_and_raise :invalid_message_content, as: InvalidSignature do (, **) end end end end |