Method: Array#split

Defined in:
lib/active_support/core_ext/array/grouping.rb

#split(value = nil) ⇒ Object

Divides the array into one or more subarrays based on a delimiting value or the result of an optional block.

[1, 2, 3, 4, 5].split(3)              # => [[1, 2], [4, 5]]
(1..10).to_a.split { |i| i % 3 == 0 } # => [[1, 2], [4, 5], [7, 8], [10]]


91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
# File 'lib/active_support/core_ext/array/grouping.rb', line 91

def split(value = nil)
  if block_given?
    inject([[]]) do |results, element|
      if yield(element)
        results << []
      else
        results.last << element
      end

      results
    end
  else
    results, arr = [[]], self.dup
    until arr.empty?
      if (idx = arr.index(value))
        results.last.concat(arr.shift(idx))
        arr.shift
        results << []
      else
        results.last.concat(arr.shift(arr.size))
      end
    end
    results
  end
end