Class: Integer

Inherits:
Object show all
Defined in:
lib/active_support/core_ext/integer/time.rb,
lib/active_support/core_ext/integer/multiple.rb,
lib/active_support/core_ext/integer/inflections.rb

Instance Method Summary collapse

Instance Method Details

#monthsObject Also known as: month

Enables the use of time calculations and declarations, like 45.minutes + 2.hours + 4.years.

These methods use Time#advance for precise date calculations when using from_now, ago, etc. as well as adding or subtracting their results from a Time object.

# equivalent to Time.now.advance(months: 1)
1.month.from_now

# equivalent to Time.now.advance(years: 2)
2.years.from_now

# equivalent to Time.now.advance(months: 4, years: 5)
(4.months + 5.years).from_now


20
21
22
# File 'lib/active_support/core_ext/integer/time.rb', line 20

def months
  ActiveSupport::Duration.new(self * 30.days, [[:months, self]])
end

#multiple_of?(number) ⇒ Boolean

Check whether the integer is evenly divisible by the argument.

0.multiple_of?(0)  # => true
6.multiple_of?(5)  # => false
10.multiple_of?(2) # => true

Returns:

  • (Boolean)


7
8
9
# File 'lib/active_support/core_ext/integer/multiple.rb', line 7

def multiple_of?(number)
  number != 0 ? self % number == 0 : zero?
end

#ordinalObject

Ordinal returns the suffix used to denote the position in an ordered sequence such as 1st, 2nd, 3rd, 4th.

1.ordinal # => "st" 2.ordinal # => "nd" 1002.ordinal # => "nd" 1003.ordinal # => "rd" -11.ordinal # => "th" -1001.ordinal # => "st"



26
27
28
# File 'lib/active_support/core_ext/integer/inflections.rb', line 26

def ordinal
  ActiveSupport::Inflector.ordinal(self)
end

#ordinalizeObject

Ordinalize turns a number into an ordinal string used to denote the position in an ordered sequence such as 1st, 2nd, 3rd, 4th.

1.ordinalize # => "1st" 2.ordinalize # => "2nd" 1002.ordinalize # => "1002nd" 1003.ordinalize # => "1003rd" -11.ordinalize # => "-11th" -1001.ordinalize # => "-1001st"



13
14
15
# File 'lib/active_support/core_ext/integer/inflections.rb', line 13

def ordinalize
  ActiveSupport::Inflector.ordinalize(self)
end

#yearsObject Also known as: year



25
26
27
# File 'lib/active_support/core_ext/integer/time.rb', line 25

def years
  ActiveSupport::Duration.new(self * 365.25.days, [[:years, self]])
end