Method: Object#`

Defined in:
lib/active_support/core_ext/kernel/agnostics.rb

#`(command) ⇒ Object

Makes backticks behave (somewhat more) similarly on all platforms. On win32 nonexistent_command raises Errno::ENOENT; on Unix, the spawned shell prints a message to stderr and sets $?. We emulate Unix on the former but not the latter.



8
9
10
11
12
# File 'lib/active_support/core_ext/kernel/agnostics.rb', line 8

def `(command) #:nodoc:
  super
rescue Errno::ENOENT => e
  STDERR.puts "#$0: #{e}"
end