Method: Object#to_query

Defined in:
lib/active_support/core_ext/object/to_query.rb

#to_query(key) ⇒ Object

Converts an object into a string suitable for use as a URL query string, using the given key as the param name.

Note: This method is defined as a default implementation for all Objects for Hash#to_query to work.



8
9
10
11
# File 'lib/active_support/core_ext/object/to_query.rb', line 8

def to_query(key)
  require 'cgi' unless defined?(CGI) && defined?(CGI::escape)
  "#{CGI.escape(key.to_param)}=#{CGI.escape(to_param.to_s)}"
end