Method: ActiveSupport::Cache::Store#clear
- Defined in:
- lib/active_support/cache.rb
#clear(options = nil) ⇒ Object
Clears the entire cache. Be careful with this method since it could affect other processes if shared cache is being used.
The options hash is passed to the underlying cache implementation.
Some implementations may not support this method.
795 796 797 |
# File 'lib/active_support/cache.rb', line 795 def clear( = nil) raise NotImplementedError.new("#{self.class.name} does not support clear") end |