Class: ActiveSupport::CurrentAttributes

Inherits:
Object
  • Object
show all
Includes:
Callbacks
Defined in:
lib/active_support/current_attributes.rb

Overview

Abstract super class that provides a thread-isolated attributes singleton, which resets automatically before and after each request. This allows you to keep all the per-request attributes easily available to the whole system.

The following full app-like example demonstrates how to use a Current class to facilitate easy access to the global, per-request attributes without passing them deeply around everywhere:

# app/models/current.rb
class Current < ActiveSupport::CurrentAttributes
attribute :account, :user
attribute :request_id, :user_agent, :ip_address

resets { Time.zone = nil }

def user=(user)
  super
  self. = user.
  Time.zone    = user.time_zone
end
end

# app/controllers/concerns/authentication.rb
module Authentication
extend ActiveSupport::Concern

included do
  before_action :authenticate
end

private
  def authenticate
    if authenticated_user = User.find_by(id: cookies.encrypted[:user_id])
      Current.user = authenticated_user
    else
      redirect_to new_session_url
    end
  end
end

# app/controllers/concerns/set_current_request_details.rb
module SetCurrentRequestDetails
extend ActiveSupport::Concern

included do
  before_action do
    Current.request_id = request.uuid
    Current.user_agent = request.user_agent
    Current.ip_address = request.ip
  end
end
end

class ApplicationController < ActionController::Base
include Authentication
include SetCurrentRequestDetails
end

class MessagesController < ApplicationController
def create
  Current..messages.create(message_params)
end
end

class Message < ApplicationRecord
belongs_to :creator, default: -> { Current.user }
after_create { |message| Event.create(record: message) }
end

class Event < ApplicationRecord
before_create do
  self.request_id = Current.request_id
  self.user_agent = Current.user_agent
  self.ip_address = Current.ip_address
end
end

A word of caution: It's easy to overdo a global singleton like Current and tangle your model as a result. Current should only be used for a few, top-level globals, like account, user, and request details. The attributes stuck in Current should be used by more or less all actions on all requests. If you start sticking controller-specific attributes in there, you're going to create a mess.

Defined Under Namespace

Modules: TestHelper

Constant Summary

Constants included from Callbacks

ActiveSupport::Callbacks::CALLBACK_FILTER_TYPES

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Callbacks

#run_callbacks

Methods included from Concern

#append_features, #class_methods, extended, #included, #prepend_features, #prepended

Constructor Details

#initializeCurrentAttributes

Returns a new instance of CurrentAttributes.



188
189
190
# File 'lib/active_support/current_attributes.rb', line 188

def initialize
  @attributes = {}
end

Instance Attribute Details

#attributesObject

Returns the value of attribute attributes.



186
187
188
# File 'lib/active_support/current_attributes.rb', line 186

def attributes
  @attributes
end

Class Method Details

.attribute(*names) ⇒ Object

Declares one or more attributes that will be given both class and instance accessor methods.



100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
# File 'lib/active_support/current_attributes.rb', line 100

def attribute(*names)
  ActiveSupport::CodeGenerator.batch(generated_attribute_methods, __FILE__, __LINE__) do |owner|
    names.each do |name|
      owner.define_cached_method(name, namespace: :current_attributes) do |batch|
        batch <<
          "def #{name}" <<
          "attributes[:#{name}]" <<
          "end"
      end
      owner.define_cached_method("#{name}=", namespace: :current_attributes) do |batch|
        batch <<
          "def #{name}=(value)" <<
          "attributes[:#{name}] = value" <<
          "end"
      end
    end
  end

  ActiveSupport::CodeGenerator.batch(singleton_class, __FILE__, __LINE__) do |owner|
    names.each do |name|
      owner.define_cached_method(name, namespace: :current_attributes_delegation) do |batch|
        batch <<
          "def #{name}" <<
          "instance.#{name}" <<
          "end"
      end
      owner.define_cached_method("#{name}=", namespace: :current_attributes_delegation) do |batch|
        batch <<
          "def #{name}=(value)" <<
          "instance.#{name} = value" <<
          "end"
      end
    end
  end
end

.before_reset(&block) ⇒ Object

Calls this block before #reset is called on the instance. Used for resetting external collaborators that depend on current values.



137
138
139
# File 'lib/active_support/current_attributes.rb', line 137

def before_reset(&block)
  set_callback :reset, :before, &block
end

.clear_allObject

:nodoc:



153
154
155
156
# File 'lib/active_support/current_attributes.rb', line 153

def clear_all # :nodoc:
  reset_all
  current_instances.clear
end

.instanceObject

Returns singleton instance for this class in this thread. If none exists, one is created.



95
96
97
# File 'lib/active_support/current_attributes.rb', line 95

def instance
  current_instances[current_instances_key] ||= new
end

.reset_allObject

:nodoc:



149
150
151
# File 'lib/active_support/current_attributes.rb', line 149

def reset_all # :nodoc:
  current_instances.each_value(&:reset)
end

.resets(&block) ⇒ Object Also known as: after_reset

Calls this block after #reset is called on the instance. Used for resetting external collaborators, like Time.zone.



142
143
144
# File 'lib/active_support/current_attributes.rb', line 142

def resets(&block)
  set_callback :reset, :after, &block
end

Instance Method Details

#resetObject

Reset all attributes. Should be called before and after actions, when used as a per-request singleton.



211
212
213
214
215
# File 'lib/active_support/current_attributes.rb', line 211

def reset
  run_callbacks :reset do
    self.attributes = {}
  end
end

#set(set_attributes) ⇒ Object

Expose one or more attributes within a block. Old values are returned after the block concludes. Example demonstrating the common use of needing to set Current attributes outside the request-cycle:

class Chat::PublicationJob < ApplicationJob
def perform(attributes, room_number, creator)
  Current.set(person: creator) do
    Chat::Publisher.publish(attributes: attributes, room_number: room_number)
  end
end
end


202
203
204
205
206
207
208
# File 'lib/active_support/current_attributes.rb', line 202

def set(set_attributes)
  old_attributes = compute_attributes(set_attributes.keys)
  assign_attributes(set_attributes)
  yield
ensure
  assign_attributes(old_attributes)
end