Method: Array#split
- Defined in:
- lib/active_support/core_ext/array/grouping.rb
#split(value = nil) ⇒ Object
Divides the array into one or more subarrays based on a delimiting value or the result of an optional block.
[1, 2, 3, 4, 5].split(3) # => [[1, 2], [4, 5]]
(1..10).to_a.split { |i| i % 3 == 0 } # => [[1, 2], [4, 5], [7, 8], [10]]
87 88 89 90 91 92 93 94 95 96 97 98 99 |
# File 'lib/active_support/core_ext/array/grouping.rb', line 87 def split(value = nil) using_block = block_given? inject([[]]) do |results, element| if (using_block && yield(element)) || (value == element) results << [] else results.last << element end results end end |