Method: String#parameterize
- Defined in:
- lib/active_support/core_ext/string/inflections.rb
#parameterize(sep = '-') ⇒ Object
Replaces special characters in a string so that it may be used as part of a 'pretty' URL.
Examples
class Person
def to_param
"#{id}-#{name.parameterize}"
end
end
@person = Person.find(1)
# => #<Person id: 1, name: "Donald E. Knuth">
<%= link_to(@person.name, person_path) %>
# => <a href="/person/1-donald-e-knuth">Donald E. Knuth</a>
154 155 156 |
# File 'lib/active_support/core_ext/string/inflections.rb', line 154 def parameterize(sep = '-') ActiveSupport::Inflector.parameterize(self, sep) end |